#ifndef _PREHTML_H
#define _PREHTML_H

#include <stdio.h>
#include <stdlib.h> // pour exit
#include <string.h>
#include <stdarg.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <ctype.h>
#include <unistd.h> // pour unlink

#include <string>
#include <vector>
using namespace std;

/* SHELL.C: interface avec le Bourne Shell*/
#define CODE_FIN_SHELL "FinShell-iopipoisqpodiopsidqpogisdfg-FinShell"
int message(const char *format, ...); // formate un message dans stderr (passe en UTF-8 si option "-u")
void erreur(const char *format, ...);
void warning(const char *format, ...);
void init_pipe(void);
void fin_pipe(void);
void envoieshell(string &s);
int recoitshell(string &s, int retireEOL);
void execshell(string &s);
void creetemp(const char *nom); /* signale la cration d'un fichier temporaire,
			   * pour permettre le nettoyage si ncessaire */

/* PREHTML.C: programme principal*/
extern int debug, structure, bits8, shell;
extern string fich_out,fich_temp;
extern const char *shell_command/*="sh"*/; // shell  utiliser (bourne-shell par dfaut)
int main(int argc,char *argv[]);

/* FICHIER.C: lecture des fichiers, inclus ou non*/
/* objet pour la lecture des fichiers*/
class fichier{
public:
  fichier() { ouvert=false; fd=NULL;}
  bool ouvert;
  long numero_ligne;
  string nom_fichier;
  FILE *fd;
  int profondeur;
};

extern fichier *fichier_courant;

void ouvrir_fichier(const char *nom); /* ouvre le premier fichier*/
void inclure_fichier(const char *nom); /* ouvre un fichier inclus*/
void dit_ou(void); /* dit o on est dans le fichier, et via quelles
		    * inclusions, pour faire message d'erreur;
		    * sort dans stderr et n'inclut pas de '\n'*/
int mongetc(void); /* lit un caractre du systme de fichier (EOF si fin)*/
void stocke(void); /* commence  enregistrer les car lurs par mongetc() */
void destocke(void); /* reprend la lecture  l'endroit signal par stocke() */
void finstocke(void); /* arrte le stockage et limine les caractres lus*/
       /* NB: on peut faire destocke et finstocke pendant la lecture de
	* caractres dj stocks par stocke(): c'est transparent */
FILE *quelfichier(void); /* dit dans quel fichier on se trouve */
off_t queloffset(void); /* donne la position actuelle dans le fichier */
int quelleligne(void); /* idem: dit  quelle ligne on est */
void revient(FILE *fich,off_t pos,int ligne);/* tente de revenir, mais vrifie*
					      * qu'on est dans le mme fichier*/

/* DECODE.C: extraction des fonctions <phtml>*/
#define EST_FONCTION (EOF!=1000?1000:1001)
extern const char *listefonctions[]; /* liste de toutes les fonctions,
  avec NULL  la fin, genre {"shell","if","fi",NULL} */
extern string chaine; /* chaine qui suit le code de la fonction */
extern string fonction; /* nom de la fonction */
string recodeHTML(string &s); /* fait les chappements HTML, et alloue de
			    * la mmoire pour la chane rsultante*/
int decode(void); /* renvoie un caractre, ou EOF si fini,
		   * ou EST_FONCTION si a trouv un fonction
		   * (ce qui demande de lire quelle_fonction() )*/

/* IMBRIQUE.C: gre les imbrications */
/* dfinition de types pour les imbrications */
typedef struct {
  int estvrai; /* valeur de la condition */
} infoif;
typedef struct {
  vector<string> liste; /* liste des valeurs que doit prendre la variable, puis NULL*/
  FILE *fichier; /* fichier o l'on est (doit tre le mme  la fin!)*/
  off_t pos; /* position dans ce fichier */
  int ligne; /* numro de la ligne courante */
  string var; /*nom de la variable qui varie*/
  size_t numval; /*numro de la valeur de la variable en cours*/
} infofor;
typedef struct {
  string condition; /* condition du while*/
  FILE *fichier; /* fichier o l'on est (doit tre le mme  la fin!)*/
  off_t pos; /* position dans ce fichier, aprs le <WHILE> */
  int ligne; /* numro de la ligne courante */
} infowhile;
typedef struct {
  vector<string> liste; /* liste des fichiers  inclure */
  int affiche; /* vrai si on devait afficher, avant de rentrer en inclusion*/
  size_t numfich; /* numro du fichier que l'on inclut */
} infoinclude;

typedef struct{
  infoif pourif;
  infofor pourfor;
  infowhile pourwhile;
  infoinclude pourinclude;
} infounion;

typedef struct _imbrique{
  enum {MAIN,IF,FOR,WHILE,INCLUDE
  } type;
  infounion info;
  struct _imbrique *precedent;
} imbrique;

extern imbrique *imbriqueactu; /* dit dans quelle imbrications nous sommes*/

const char *typeimbrique(imbrique *i); /* donne le nom d'une imbrication */
int attend(const char *etiq); /* trouve le champ "etiq" correspondant sans
			  * excuter le code intermdiaire, mais en
			  * faisant attention aux imbrications*/
  /* si etiq="else", renvoie 1 si trouv, et 0 si a rencontr "fi" */
void referme_imbrique(void); /* referme l'imbrication courant */
void ouvre_imbrique(void); /* operations gnriques pour tous les types */

/* FONCTION.C: quand on a reu une fonction, fait presque tout */
extern int flag_reinclude; /* le prochain fichier inclus doit-il tre
			      * inclus avec son entte, remplaant l'ancien? */
extern int affiche; /* vrai si on doit afficher ce qu'on lit*/
extern int chut; /* vrai si on a cache un morceau */
extern FILE *sortie;
void print(const char *format, ...); /* crit dans le fichier cible (sortie) */
void put(char c); /* idem, mais imite putc() au lieu de fprintf */
int vraifaux(const char *commande, const char *condition); /* utilis par IF et WHILE */
void traitefonction(void); /* fait presque tout */

/* GENERATE.C: gestion de <generate> et </generate> */
typedef struct structgenerate{
  string nomtemp;
  string nom;
} generate;

extern int engenerate; /* 0=pas de gnrate, 1=dans gnrate
			* 2=generate pass */
void init_generate(void);
void genere(const char *s); /* correspond  <GENERATE> */
void degenere(void); /* fonction appele quand </generate> arrive */
void fin_generate(void); /* choses  faire  la fin du fichier principal pour
			  * finaliser les <GENERATE> ventuels */

#endif //#ifdef _PREHTML_H

